package net.ajaest.lib.swing.util;

import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.JTextComponent;

import net.ajaest.lib.locale.SwingMessages;
import net.ajaest.lib.locale.SwingMessagesEnums;
import net.ajaest.lib.locale.SwingMessagesEnums.ContextMenuEnums.EditMenu;
import net.ajaest.lib.swing.util.action.JComboBoxCopyTextAction;
import net.ajaest.lib.swing.util.action.JLabelCopyTextAction;
import net.ajaest.lib.swing.util.action.JListCopyTextAction;
import net.ajaest.lib.utils.SystemEnums.ISO639ー1;

public class EditContextMenuEngine {

	private class ContextMenuMouseListener extends MouseAdapter {

		private final EditContextMenuEngine ecm;

		private ContextMenuMouseListener(EditContextMenuEngine ecm) {
			this.ecm = ecm;
		}

		@Override
		public void mousePressed(MouseEvent e) {
			this.showContextMenu(e);
		}

		@Override
		public void mouseReleased(MouseEvent e) {
			this.showContextMenu(e);
		}

		private void showContextMenu(MouseEvent e) {

			JPopupMenu jpm = this.ecm.getContextMenu(e.getComponent());

			if (jpm == null) {
				jpm = this.ecm.createDefaultEditContextMenu(e.getComponent(), null, null);
				this.ecm.menus.put(e.getComponent(), jpm);
			}

			if (e.isPopupTrigger() && (jpm != null)) {
				e.getComponent().requestFocus();
				jpm.show(e.getComponent(), e.getX(), e.getY());
				jpm.setVisible(true);
			}
		}
	}

	private static final long serialVersionUID = -5221367951857450395L;
	private final Map<Component, JPopupMenu> menus;
	private final Map<SwingMessagesEnums.ContextMenuEnums.EditMenu, String> messages;

	private final ContextMenuMouseListener cmml;

	public EditContextMenuEngine(ISO639ー1 messageLang) {
		this.messages = SwingMessages.getLocalizedMessages(messageLang).getContextMenuEditMessages();
		this.menus = new HashMap<Component, JPopupMenu>();
		this.cmml = new ContextMenuMouseListener(this);
	}

	/**
	 * Adds a default edit context menu to the specified component.
	 * 
	 * @param c
	 *            the component to which context menu will be added.
	 * @return the added {@code JPopupMenu}, null if no context menu were added.
	 */
	public JPopupMenu addContextMenu(JComponent c) {
		JPopupMenu jpm = this.addContextMenu(c, null, null);

		return jpm;
	}

	/**
	 * Adds a default edit context menu to the specified component in which the
	 * default action associated to an edit menu action is replaced by the given
	 * action. If the given action is null no replace will be done.
	 * 
	 * @param c
	 *            the component to which context menu will be added.
	 * @param em
	 *            the edit menu action to be replaced
	 * @param atc
	 *            the action that will replace the selected default edit menu
	 *            action
	 * @return the added {@code JPopupMenu}, null if no context menu were added.
	 */
	public JPopupMenu addContextMenu(JComponent c, EditMenu em, Action atc) {

		JPopupMenu jpm = this.createDefaultEditContextMenu(c, em, atc);

		if (jpm != null) {
			c.add(jpm);
			this.menus.put(c, jpm);
		}

		return jpm;
	}

	/**
	 * Adds manually an edit context menu to the specified component.
	 * 
	 * @param c
	 *            the component to which context menu will be added.
	 * @param jpm
	 *            the pop menu to be added
	 * @return the added {@code JPopupMenu}, null if no context menu were added.
	 */
	public JPopupMenu addContextMenu(JComponent c, JPopupMenu jpm) {

		if (jpm != null) {
			c.add(jpm);
			this.menus.put(c, jpm);
		}

		return jpm;
	}

	private Action checkReplacement(EditMenu emDefault, Action atcDefault, EditMenu emReplace, Action atcReplace) {

		if ((emReplace != null) && (emDefault != null) && emDefault.equals(emReplace)) {
			if (atcReplace != null)
				return atcReplace;
			else
				return atcDefault;
		} else {
			return atcDefault;
		}
	}

	/**
	 * Returns {@code null} if no default edit context menu can be created
	 */
	private JPopupMenu createDefaultEditContextMenu(Component c, EditMenu em, Action atc) {

		JPopupMenu jpm = null;

		if (c instanceof JTextComponent) {

			jpm = new JPopupMenu();

			JTextComponent jtc = (JTextComponent) c;

			ActionMap actionMap = jtc.getActionMap();
			Action copyAction = this.checkReplacement(EditMenu.COPY, actionMap.get(DefaultEditorKit.copyAction), em, atc);
			Action pasteAction = this.checkReplacement(EditMenu.PASTE, actionMap.get(DefaultEditorKit.pasteAction), em, atc);
			Action cutAction = this.checkReplacement(EditMenu.CUT, actionMap.get(DefaultEditorKit.cutAction), em, atc);
			Action selectAllAction = this.checkReplacement(EditMenu.SELECT_ALL, actionMap.get(DefaultEditorKit.selectAllAction), em, atc);

			JMenuItem jmCut = new JMenuItem(cutAction);
			jmCut.setText(this.messages.get(EditMenu.CUT));
			jpm.add(jmCut);

			JMenuItem jmCopy = new JMenuItem(copyAction);
			jmCopy.setText(this.messages.get(EditMenu.COPY));
			jpm.add(jmCopy);

			JMenuItem jmPaste = new JMenuItem(pasteAction);
			jmPaste.setText(this.messages.get(EditMenu.PASTE));
			jpm.add(jmPaste);

			JMenuItem jmSelectAll = new JMenuItem(selectAllAction);
			jmSelectAll.setText(this.messages.get(EditMenu.SELECT_ALL));
			jpm.add(jmSelectAll);

			jtc.addMouseListener(this.cmml);

		} else if(c instanceof JLabel) {

			jpm = new JPopupMenu();
			JLabel jl = (JLabel)c;

			Action copyAction = this.checkReplacement(EditMenu.COPY, new JLabelCopyTextAction(jl), em, atc);

			JMenuItem jmCopy = new JMenuItem(copyAction);
			jmCopy.setText(this.messages.get(EditMenu.COPY));
			jpm.add(jmCopy);

			jl.addMouseListener(this.cmml);

		} else if (c instanceof JComboBox) {

			jpm = new JPopupMenu();

			JComboBox jcb = (JComboBox) c;

			Action copyAction = this.checkReplacement(EditMenu.COPY, new JComboBoxCopyTextAction(jcb), em, atc);

			JMenuItem jmCopy = new JMenuItem(copyAction);
			jmCopy.setText(this.messages.get(EditMenu.COPY));
			jpm.add(jmCopy);

			JTextComponent editableCBComp = (JTextComponent) jcb.getEditor().getEditorComponent();

			editableCBComp.addMouseListener(this.cmml);
			jcb.addMouseListener(this.cmml);
		} else if (c instanceof JList) {

			jpm = new JPopupMenu();

			JList jl = (JList) c;

			Action copyAction = this.checkReplacement(EditMenu.COPY, new JListCopyTextAction(jl, "\r\n"), em, atc);

			JMenuItem jmCopy = new JMenuItem(copyAction);
			jmCopy.setText(this.messages.get(EditMenu.COPY));
			jpm.add(jmCopy);

			jl.addMouseListener(this.cmml);
		}

		return jpm;
	}

	public JPopupMenu getContextMenu(Component c) {
		return this.menus.get(c);
	}
}
