//    Copyright (C) 2011  Luis Alfonso Arce González, ajaest[@]gmail.com
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package net.ajaest.lib.swing.window;

import java.awt.BorderLayout;
import java.awt.Desktop;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * <p>
 * This class can be used to display a message and a moving bar to the user
 * while a heavy task that requires the whole application to be blocked is being
 * performed. The task will begin if the any of the methods
 * {@link #setVisible(boolean)} or {@link #run()} are called.
 * </p>
 * <p>
 * The user won't be able to close the dialog whatever default close operation
 * was set. However, the default close operation set by the user will be
 * restored/applied once the task is finished
 * </p>
 * <p>
 * When the task execution is finished, another message and an accept button are
 * displayed. The default action of accept button is to dispose the window, but
 * if a {@link Runnable} instance is provided through
 * {@link #setAcceptAction(Runnable)}, that action will be performed whe the
 * button is pressed.
 * </p>
 * <p>
 * Furthermore, this class provides techniques that helps the user thinks that
 * the application isn't frozen. For example, setting an alternative title
 * string calling {@link #setUserImpatienceTitle(String)}, the dialog can change it's own
 * title if the user presses window's close button during the task execution.
 * </p>
 * <p>
 * An alternative string title displayed when the task is finished can be set
 * calling the {@link #setFinishedTitle(String)} method.
 * </p>
 *
 * @author Luis A. Arce
 *
 */
public class LoadingDialog extends JDialog implements Runnable{

    private static final long serialVersionUID = -941470170052629278L;

    private final Runnable    task;
    private       boolean     executingTask;

    private       JTextArea   loadingLabel;
    private       JComponent  finishedPanel;
    private       JScrollPane loadingScrollPane;
    private       JLabel      loadingImageLabel;

    private       JButton     acceptButton;
    private       Runnable    acceptAction = null;

    private       String      title = null;
    private       String      userImpatienceTitle = null;
    private       String      finishedTitle = null;

    private       Integer     defaultCloseOperation;

    private Exception throwedByRunnable = null;

    public LoadingDialog(String loadingText, String finishedText, String acceptText, Runnable task, Window owner){
        this(loadingText, (JComponent)null, acceptText, task, owner);

        this.setFinishedText(finishedText);
    }

    public LoadingDialog(String loadingText, JComponent finishedPanel,String acceptText, Runnable loadingAction, Window owner){
        super(owner, ModalityType.APPLICATION_MODAL);

        this.initComponents(owner,loadingText, acceptText);
        this.task = loadingAction;
        this.finishedPanel = finishedPanel;
        this.defaultCloseOperation = super.getDefaultCloseOperation();
    }

    public void setFinishedText(String finishedText){
        JTextArea finishedLabel = new JTextArea(finishedText);
        finishedLabel.setOpaque(false);
        finishedLabel.setEditable(false);
        finishedLabel.setHighlighter(null);
        finishedLabel.setLineWrap(true);
        finishedLabel.setWrapStyleWord(true);
        finishedLabel.setMargin(new Insets(20,20,20,20));
        this.finishedPanel = finishedLabel;
    }

    public void setAcceptButtonText(String text) {
        this.acceptButton.setText(text);
    }

    private void initComponents(Window owner, String loadingText, String acceptText){

        ImageIcon loadingIcon = new ImageIcon(this.getClass().getResource("ajax-loader.gif"));

        this.setLayout(new BorderLayout());
        this.addWindowListener(new TitleChangerWindowAdapter());
        this.setResizable(false);
        this.setSize(loadingIcon.getIconWidth(), loadingIcon.getIconHeight()*6);
        this.setLocationRelativeTo(owner);

        this.loadingLabel = new JTextArea(loadingText);
        this.loadingLabel.setOpaque(false);
        this.loadingLabel.setEditable(false);
        this.loadingLabel.setHighlighter(null);
        this.loadingLabel.setLineWrap(true);
        this.loadingLabel.setWrapStyleWord(true);
        this.loadingLabel.setMargin(new Insets(20,20,20,20));


        this.loadingScrollPane = new JScrollPane();
        this.loadingScrollPane.setBorder(null);
        this.loadingScrollPane.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        this.loadingScrollPane.setViewportBorder(null);
        this.loadingScrollPane.setOpaque(false);
        this.loadingScrollPane.getViewport().setOpaque(false);
        this.loadingScrollPane.setViewportView(this.loadingLabel);

        this.add(this.loadingScrollPane,BorderLayout.CENTER);

        this.loadingImageLabel = new JLabel(loadingIcon);
        this.add(this.loadingImageLabel, BorderLayout.SOUTH);

        this.acceptButton = new JButton(acceptText);
        final LoadingDialog ref = this;
        this.acceptButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if(ref.acceptAction!=null)
                    new Thread(ref.acceptAction).start();
                ref.dispose();
            }
        });
    }

    /**
     *
     * @return the exception thrown by the runnable, {@code null} if no
     *         exception were thrown
     */
    public Exception getExceptionThrowedByRunnable() {
        return this.throwedByRunnable;
    }

    @Override
    public void setVisible(boolean b){

        new DialogUpdater(this).start();
        super.setVisible(b);
    }

    private void updateFinished(){

        if(this.finishedPanel!=null){
            this.loadingScrollPane.remove(this.loadingLabel);
            this.loadingScrollPane.setViewportView(this.finishedPanel);
        }

        this.remove(this.loadingImageLabel);
        this.add(this.acceptButton, BorderLayout.SOUTH);

        if(this.finishedTitle!=null)
            super.setTitle(this.finishedTitle);
        else if(this.title!=null)
            super.setTitle(this.title);

        this.validate();
        this.repaint();
    }

    @Override
    public void run() {
        this.setVisible(true);
    }

    public void setUserImpatienceTitle(String text) {
        this.userImpatienceTitle = text;
    }

    private void disableClosingOperation(){
        super.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
    }

    private void restoreClosingOperation(){
        super.setDefaultCloseOperation(this.defaultCloseOperation);
    }

    @Override
    public void setDefaultCloseOperation(int operation) {
        this.defaultCloseOperation = operation;
        if(!this.executingTask)
            super.setDefaultCloseOperation(operation);
    }


    private class TitleChangerWindowAdapter extends WindowAdapter{
        public TitleChangerWindowAdapter(){}
        @Override
        public void windowClosing(WindowEvent e) {
            if(e.getWindow() instanceof LoadingDialog){
                LoadingDialog ld = (LoadingDialog)e.getWindow();

                if(ld.executingTask){
                    // An "I'm alive!" message is displayed if the user press
                    // window's close button before finishing task
                    if(LoadingDialog.this.userImpatienceTitle!=null)
                        ld.setImpatienceTitle();
                } else {
                    // If the task is finished, the dialog must be prepared for
                    // next call
                    ld.remove(ld.acceptButton);

                    ld.loadingScrollPane.setViewportView(ld.loadingLabel);
                    ld.add(ld.loadingImageLabel, BorderLayout.SOUTH);
                    ld.setTitle(ld.title);
                }
            }
        }
    }

    private class DialogUpdater extends Thread{

        private final LoadingDialog ld;

        private DialogUpdater(LoadingDialog ld){
            super("DialogUpdater");
            this.ld = ld;

        }

        @Override
        public void run() {
            this.ld.executingTask = true;
            this.ld.disableClosingOperation();

            try {
                this.ld.task.run();
            } catch (Exception e) {
                this.ld.setFinishedTitle("Error: " + e.getClass().getName());
                this.ld.setFinishedText(e.getLocalizedMessage());
                this.ld.throwedByRunnable = e;
            }

            this.ld.restoreClosingOperation();
            this.ld.executingTask = false;

            this.ld.updateFinished();
        }
    }

    public void setAcceptAction(Runnable acceptAction){
        this.acceptAction = acceptAction;
    }

    private void setImpatienceTitle() {
        super.setTitle(this.userImpatienceTitle);
    }

    public void setFinishedTitle(String finishedText) {
        this.finishedTitle = finishedText;
    }

    public String getFinishedTitle() {
        return this.finishedTitle;
    }

    public String getUserImpatienceTitle() {
        return this.userImpatienceTitle;
    }

    @Override
    public void setTitle(String title){
        this.title = title;
        super.setTitle(title);
    }

    public static void main(String...args){
        Runnable task = new Runnable() {
            @Override
            public void run() {
                try {
                    for(int i=0; i<10; i++){
                        System.out.println(i);
                        Thread.sleep(500);
                    }
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                System.out.println("Terminado");
                throw new RuntimeException("WTF!");

            }
        };

        Runnable openFile = new Runnable() {

            @Override
            public void run() {
                Desktop d = Desktop.getDesktop();
                try{
                    d.open(new File("/home/ajaest11/Escritorio/asd1.pdf"));
                }catch (Exception e) {}
            }
        };



        JFrame jf = new JFrame();
        jf.setSize(200,200);
        jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        final LoadingDialog ld = new LoadingDialog("Cargando...", "Cargado","Aceptar", task, jf);
        ld.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        ld.setAcceptAction(openFile);
        ld.setTitle("Exportando");
        ld.setFinishedTitle("Terminado");
        ld.setUserImpatienceTitle("Espere...");
        //ld.setSize(300, 180);

        JButton run = new JButton("Run");
        run.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                ld.setVisible(true);

            }
        });

        jf.add(run);
        jf.setVisible(true);
    }
}
